import _Promise from 'babel-runtime/core-js/promise';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';

var TaskQueue = function () {
  function TaskQueue() {
    _classCallCheck(this, TaskQueue);

    this.queue = {};
  }

  _createClass(TaskQueue, [{
    key: 'push',
    value: function push(_ref) {
      var url = _ref.url,
          task = _ref.task;

      var taskQueue = this.queue[url];
      if (taskQueue === undefined || taskQueue === null) {
        this.queue[url] = taskQueue = [];
      }
      taskQueue.push(task);
    }
  }, {
    key: 'get',
    value: function get(url) {
      var taskQueue = this.queue[url];
      if (taskQueue !== undefined && taskQueue !== null) {
        return taskQueue.shift();
      } else {
        return null;
      }
    }
  }]);

  return TaskQueue;
}();

var taskQueue = new TaskQueue(); // 定义一个操作队列，用于储存script没有完成加载时的任务

var createScript = function createScript(url) {
  var script = document.createElement('script');
  script.type = 'text/javascript';
  script.src = url;
  return script;
};

var scriptExist = function scriptExist(url) {
  return document.querySelector('script[src=\'' + url + '\']') !== null;
};

var scriptLoaded = function scriptLoaded(url) {
  // 检测脚本是否已经加载完成
  return scriptExist(url) && document.querySelector('script[src=\'' + url + '\']').getAttribute('data-load-state') === 'loaded';
};

/**
 * 一个脚本加载器，用于动态脚本加载，基于Promise
 */
export default {
  loadScript: function loadScript(url) {
    return new _Promise(function (resolve, reject) {
      if (scriptLoaded(url)) {
        // 判断某个script是否已经加载
        resolve(); // 如果加载，直接返回
      } else {
        taskQueue.push({ url: url, task: resolve }); // 如果没有加载，将任务压入队列
        if (!scriptExist(url)) {
          // 判断script是否已经插入document,如果没有插入document,这执行插入动作
          var script = createScript(url);
          script.setAttribute('data-load-state', 'loading');
          document.body.appendChild(script);
          var onScriptLoad = function onScriptLoad() {
            // 定义script加载完成之后的操作
            script.removeEventListener('load', onScriptLoad);
            script.setAttribute('data-load-state', 'loaded');
            // 获取队列里面的任务，依次执行
            var fn = void 0;
            while ((fn = taskQueue.get(url)) !== undefined && fn !== null) {
              fn();
            }
          };
          script.addEventListener('load', onScriptLoad);
        }
      }
    });
  }
};